#ifdef FEATURE_TEMP

#include "OneWire.h"

#define ONE_WIRE_BUS      17
#define ID_SENSORS        8
#define FULL_HEALTH       100
#define ZERO_HEALTH       0
#define MAX_TEMP_NAME_LEN 12
#define TEMP_SENSORS      3    // количество датчиков
#define CYCLE_TIME        60   // частота измерений температуры(в секундах)

byte tempSensorHealth[TEMP_SENSORS];
byte tempModuleHealth;

char objSens[TEMP_SENSORS][MAX_TEMP_NAME_LEN] = {"tempSTR", "tempHOM", "tempWRM"};

// Sensors ID's
byte idTempDat[TEMP_SENSORS][ID_SENSORS] = {
  //{40, 255, 237, 27, 3, 22, 3},
  //{40, 255, 237, 27, 3, 22, 3},
  //{40, 255, 237, 27, 3, 22, 3},
  {40, 156, 93, 205, 5, 0, 0, 58},     /// адрес датчика !!!
  {40, 156, 93, 205, 5, 0, 0, 58},   ///
  {40, 156, 93, 205, 5, 0, 0, 58}    ///
}; 

float current_temp[TEMP_SENSORS];
int temp_[TEMP_SENSORS];

OneWire oneWire(ONE_WIRE_BUS);

bool flag = 0;
unsigned long convert = 0;
unsigned long timer = 0;

void initTemp() {

//  initStart("Temperature");
  timeStamp();
  search_Sensors_Id();
  show_Sensors_Conf();
  
  moduleTemp = ENABLE;
//  initDone();

  started(F("Temp"), true);
}

void show_Sensors_Conf() {
  for (byte n = 0; n < TEMP_SENSORS; n++) {
    timeStamp();
    Serial.print(F("#"));
    Serial.print(n);
    Serial.print(F(":"));
    for (byte i = 0; i < ID_SENSORS; i++) {
      Serial.print(" ");
      Serial.print(idTempDat[n][i]);
    }
    Serial.print(" ");
    Serial.println(objSens[n]);
  }
}

void search_Sensors_Id() {
  int count = 0;
  unsigned char addr[8];
  #ifdef ESP32
  portDISABLE_INTERRUPTS();
  #endif
  for (int i=0; ; i++) {
    if (oneWire.search(addr)) {
      count++;
    //  #ifdef TEMP_DEBUG//===========================================================
        Serial.print("#"); Serial.print(i); Serial.print(": ");
        for (int j = 0; j < 8; j++) {
          Serial.print(addr[j]); Serial.print(" ");
        }
        Serial.println();
     // #endif // TEMP_DEBUG//=====================================
    } else {
      oneWire.reset_search();
      break;
    }
  }
  #ifdef ESP32
  portENABLE_INTERRUPTS();
  #endif
  timeStamp();
  if (count > 0) {
    Serial.print(F("Found ")); Serial.print(count); Serial.println(F(" devices"));
  } else { Serial.print(F("Devices no found"));  }
}

void setModuleTempHealth() {
  int sumSensorsHealth = 0;
  for (byte i = 0; i < TEMP_SENSORS; i++) {
    sumSensorsHealth += tempSensorHealth[i];
  }
  tempModuleHealth = sumSensorsHealth / TEMP_SENSORS;
}

void workTemp() {
  if (millis() > timer) {
    if (!flag){
      oneWire.reset(); 
      oneWire.skip();
    //  oneWire.write(0x44);//=======================================
     oneWire.write(0x44, 1);  // паразитное питание
      flag = 1;
      convert = millis() + 1000;
    }
    else if (millis() > convert) {
      for (byte i = 0; i < TEMP_SENSORS; i++) {
        byte data[9];
        #ifdef ESP32
        portDISABLE_INTERRUPTS();
        #endif
        oneWire.reset();
        oneWire.select(idTempDat[i]);
        oneWire.write(0xBE);
        for (byte x = 0; x < 9; x++) {
          data[x] = oneWire.read();
        }
        #ifdef ESP32
        portENABLE_INTERRUPTS();
        #endif
        if (data[0] != 0) {
          int16_t raw = (data[1] << 8) | data[0];
          current_temp[i] = raw / 16.0;
          if (current_temp[i] != -127) {
            if (current_temp[i] != 85) {
              if (OneWire::crc8(data, 8) == data[8]) {
                timeStamp();
                Serial.print(objSens[i]);
                Serial.print(": ");
                Serial.println(current_temp[i]);//===============================================
                tempSensorHealth[i] = FULL_HEALTH;
                temp_[i] = (current_temp[i] - (int)current_temp[i]) * 100;
                if ((current_temp[i] > -1) && (current_temp[i] < 0)) {
                  #ifdef FEATURE_MAJOR
                    sendRequestMinusM(objSens[i], "-", (int)current_temp[i], abs(temp_[i])); 
                  #endif
                } else {
                  #ifdef FEATURE_MAJOR
                    sendRequestMinusM(objSens[i], "", (int)current_temp[i], abs(temp_[i])); 
                  #endif
                }
              } else { // OneWire::crc8 !=
                tempSensorHealth[i] = ZERO_HEALTH;
                timeStamp();
                Serial.print(objSens[i]);
                Serial.print(": ");
                Serial.println(F("CRC error"));
              }
            } else { // current_temp[i] = 85
              tempSensorHealth[i] = ZERO_HEALTH;
              timeStamp();
              Serial.print(objSens[i]);
              Serial.print(": ");
              Serial.println(F("error")); 
            }
          } else { // current_temp[i] = -127
            tempSensorHealth[i] = ZERO_HEALTH;
            timeStamp();
            Serial.print(objSens[i]);
            Serial.print(": ");
            Serial.println(F("removed")); 
          }
        } else  { // current Sensor ID != stored Sensor ID
          tempSensorHealth[i] = ZERO_HEALTH;
          timeStamp();
          Serial.print(objSens[i]);
          Serial.print(": ");
          Serial.println(F("ID problem or not present"));
        }
      }
      flag = 0;
      timer = millis() + (CYCLE_TIME * 1000);
      #ifdef TEMP_DEBUG
      for (byte i = 0; i < TEMP_SENSORS; i++) {
        timeStamp();
        Serial.print(F("Sensor "));
        Serial.println(objSens[i]);//==========
        Serial.print(F(" health: "));
        Serial.println(tempSensorHealth[i]);
      }
      setModuleTempHealth();
      timeStamp();
      Serial.print(F("Modile Temperature Health: "));
      Serial.println(tempModuleHealth);
      #endif // TEMP_DEBUG
    }
  }
}

#endif // FEATURE_TEMP
